@@ -65,7 +65,8 @@ typedef struct status_table_struct {
 // Derived from:
 // gperf -7 --language=ANSI-C --key-positions=1,3,4 -C -n -c sml.gperf
 
-static void status2proc(char *S, proc_t *restrict P){
+static void status2proc(char *S, proc_t *restrict P, int is_proc){
+    char ShdPnd[16] = "";
     static const unsigned char asso[] = {
         56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56,
         56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56,
@@ -215,10 +216,13 @@ ENTER(0x220);
     case_Pid:
         P->tid = strtol(S,&S,10);
         continue;
+    case_Threads:
+        P->nlwp = strtol(S,&S,10);
+        continue;
 
     case_ShdPnd:
-        memcpy(P->signal, S, 16);
-        P->signal[16] = '\0';
+        memcpy(ShdPnd, S, 16);
+        // we know it to be 16 char, so no '\0' needed
         continue;
     case_SigBlk:
         memcpy(P->blocked, S, 16);
@@ -271,6 +275,13 @@ ENTER(0x220);
         P->vm_stack = strtol(S,&S,10);
         continue;
     }
+
+    // recent kernels supply per-tgid pending signals
+    if(is_proc && *ShdPnd){
+	memcpy(P->signal, ShdPnd, 16);
+	P->signal[16] = '\0';
+    }
+
 LEAVE(0x220);
 }
 
@@ -303,7 +314,9 @@ ENTER(0x160);
        "%d %d %d %d %d "
        "%lu %lu %lu %lu %lu "
        "%Lu %Lu %Lu %Lu "  /* utime stime cutime cstime */
-       "%ld %ld %ld %ld "
+       "%ld %ld "
+       "%d "
+       "%ld "
        "%Lu "  /* start_time */
        "%lu "
        "%ld "
@@ -316,7 +329,9 @@ ENTER(0x160);
        &P->ppid, &P->pgrp, &P->session, &P->tty, &P->tpgid,
        &P->flags, &P->min_flt, &P->cmin_flt, &P->maj_flt, &P->cmaj_flt,
        &P->utime, &P->stime, &P->cutime, &P->cstime,
-       &P->priority, &P->nice, &P->timeout, &P->it_real_value,
+       &P->priority, &P->nice,
+       &P->nlwp,
+       &P->it_real_value,
        &P->start_time,
        &P->vsize,
        &P->rss,
@@ -494,7 +509,7 @@ static proc_t* simple_readproc(PROCTAB *restrict const PT, proc_t *restrict cons
 
     if (flags & PROC_FILLSTATUS) {         /* read, parse /proc/#/status */
        if (likely( file2str(path, "status", sbuf, sizeof sbuf) != -1 )){
-           status2proc(sbuf, p);
+           status2proc(sbuf, p, 1);
        }
     }
 
@@ -543,6 +558,7 @@ static proc_t* simple_readtask(PROCTAB *restrict const PT, const proc_t *restric
     static char sbuf[1024];	// buffer for stat,statm
     unsigned flags = PT->flags;
 
+//printf("hhh\n");
     if (unlikely(stat(path, &sb) == -1))	/* no such dirent (anymore) */
 	goto next_task;
 
@@ -552,6 +568,7 @@ static proc_t* simple_readtask(PROCTAB *restrict const PT, const proc_t *restric
     t->euid = sb.st_uid;			/* need a way to get real uid */
     t->egid = sb.st_gid;			/* need a way to get real gid */
 
+//printf("iii\n");
     if (flags & PROC_FILLSTAT) {         /* read, parse /proc/#/stat */
 	if (unlikely( file2str(path, "stat", sbuf, sizeof sbuf) == -1 ))
 	    goto next_task;			/* error reading /proc/#/stat */
@@ -559,13 +576,23 @@ static proc_t* simple_readtask(PROCTAB *restrict const PT, const proc_t *restric
     }
 
     if (unlikely(flags & PROC_FILLMEM)) {	/* read, parse /proc/#/statm */
+#if 0
 	if (likely( file2str(path, "statm", sbuf, sizeof sbuf) != -1 ))
 	    statm2proc(sbuf, t);		/* ignore statm errors here */
+#else
+	t->size     = p->size;
+	t->resident = p->resident;
+	t->share    = p->share;
+	t->trs      = p->trs;
+	t->lrs      = p->lrs;
+	t->drs      = p->drs;
+	t->dt       = p->dt;
+#endif
     }						/* statm fields just zero */
 
     if (flags & PROC_FILLSTATUS) {         /* read, parse /proc/#/status */
        if (likely( file2str(path, "status", sbuf, sizeof sbuf) != -1 )){
-           status2proc(sbuf, t);
+           status2proc(sbuf, t, 0);
        }
     }
 
@@ -604,6 +631,8 @@ static proc_t* simple_readtask(PROCTAB *restrict const PT, const proc_t *restric
     t->environ = p->environ;
 #endif
 
+    t->ppid = p->ppid;  // ought to put the per-task ppid somewhere
+
     return t;
 next_task:
     return NULL;
@@ -632,12 +661,15 @@ static int simple_nextpid(PROCTAB *restrict const PT, proc_t *restrict const p)
 // Return non-zero on success.
 static int simple_nexttid(PROCTAB *restrict const PT, const proc_t *restrict const p, proc_t *restrict const t, char *restrict const path) {
   static struct direct *ent;		/* dirent handle */
-  (void)p;
-  if(!PT->taskdir){
+  if(PT->taskdir_user != p->tgid){
+    if(PT->taskdir){
+      closedir(PT->taskdir);
+    }
     // use "path" as some tmp space
-    snprintf(path, PROCPATHLEN, "%s/task", PT->path);
+    snprintf(path, PROCPATHLEN, "/proc/%d/task", p->tgid);
     PT->taskdir = opendir(path);
     if(!PT->taskdir) return 0;
+    PT->taskdir_user = p->tgid;
   }
   for (;;) {
     ent = readdir(PT->taskdir);
@@ -647,7 +679,7 @@ static int simple_nexttid(PROCTAB *restrict const PT, const proc_t *restrict con
   t->tid = strtoul(ent->d_name, NULL, 10);
   t->tgid = p->tgid;
   t->ppid = p->ppid;  // cover for kernel behavior? we want both actually...?
-  snprintf(path, PROCPATHLEN, "%s/task/%s", PT->path, ent->d_name);
+  snprintf(path, PROCPATHLEN, "/proc/%d/task/%s", p->tgid, ent->d_name);
   return 1;
 }
 
@@ -734,10 +766,11 @@ proc_t* readproc(PROCTAB *restrict const PT, proc_t *restrict p) {
   proc_t *saved_p;
 
   if (PT->did_fake) PT->did_fake=0;
-  if (PT->taskdir) {
-    closedir(PT->taskdir);
-    PT->taskdir = NULL;
-  }
+//  if (PT->taskdir) {
+//    closedir(PT->taskdir);
+//    PT->taskdir = NULL;
+//    PT->taskdir_user = -1;
+//  }
 
   saved_p = p;
   if(!p) p = xcalloc(p, sizeof *p); /* passed buf or alloced mem */
@@ -841,6 +874,7 @@ PROCTAB* openproc(int flags, ...) {
       did_stat = 1;
     }
     PT->taskdir = NULL;
+    PT->taskdir_user = -1;
     PT->taskfinder = simple_nexttid;
     PT->taskreader = simple_readtask;
 
@@ -883,6 +917,7 @@ void closeproc(PROCTAB* PT) {
     if (PT){
         if (PT->procfs) closedir(PT->procfs);
         if (PT->taskdir) closedir(PT->taskdir);
+        memset(PT,'#',sizeof(PROCTAB));
         free(PT);
     }
 }
@@ -913,6 +948,7 @@ void look_up_our_self(proc_t *p) {
 }
 
 HIDDEN_ALIAS(readproc);
+HIDDEN_ALIAS(readtask);
 
 /* Convenient wrapper around openproc and readproc to slurp in the whole process
  * table subset satisfying the constraints of flags and the optional PID list.
@@ -949,3 +985,79 @@ proc_t** readproctab(int flags, ...) {
     closeproc(PT);
     return tab;
 }
+
+// Try again, this time with threads and selection.
+proc_data_t *readproctab2(int(*want_proc)(proc_t *buf), int(*want_task)(proc_t *buf), PROCTAB *restrict const PT) {
+    proc_t** ptab = NULL;
+    unsigned n_proc_alloc = 0;
+    unsigned n_proc = 0;
+
+    proc_t** ttab = NULL;
+    unsigned n_task_alloc = 0;
+    unsigned n_task = 0;
+
+    proc_t*  data = NULL;
+    unsigned n_alloc = 0;
+    unsigned long n_used = 0;
+
+    proc_data_t *pd;
+
+    for(;;){
+        proc_t *tmp;
+        if(n_alloc == n_used){
+          //proc_t *old = data;
+          n_alloc = n_alloc*5/4+30;  // grow by over 25%
+          data = realloc(data,sizeof(proc_t)*n_alloc);
+          //if(!data) return NULL;
+        }
+        if(n_proc_alloc == n_proc){
+          //proc_t **old = ptab;
+          n_proc_alloc = n_proc_alloc*5/4+30;  // grow by over 25%
+          ptab = realloc(ptab,sizeof(proc_t*)*n_proc_alloc);
+          //if(!ptab) return NULL;
+        }
+        tmp = readproc_direct(PT, data+n_used);
+        if(!tmp) break;
+        if(!want_proc(tmp)) continue;
+        ptab[n_proc++] = (proc_t*)(n_used++);
+        if(!(  PT->flags & PROC_LOOSE_TASKS  )) continue;
+        for(;;){
+          proc_t *t;
+          if(n_alloc == n_used){
+            //proc_t *old = data;
+            n_alloc = n_alloc*5/4+30;  // grow by over 25%
+            data = realloc(data,sizeof(proc_t)*n_alloc);
+            //if(!data) return NULL;
+          }
+          if(n_task_alloc == n_task){
+            //proc_t **old = ttab;
+            n_task_alloc = n_task_alloc*5/4+1;  // grow by over 25%
+            ttab = realloc(ttab,sizeof(proc_t*)*n_task_alloc);
+            //if(!ttab) return NULL;
+          }
+          t = readtask_direct(PT, tmp, data+n_used);
+          if(!t) break;
+          if(!want_task(t)) continue;
+          ttab[n_task++] = (proc_t*)(n_used++);
+        }
+    }
+
+    pd = malloc(sizeof(proc_data_t));
+    pd->proc = ptab;
+    pd->task = ttab;
+    pd->nproc = n_proc;
+    pd->ntask = n_task;
+    if(PT->flags & PROC_LOOSE_TASKS){
+      pd->tab = ttab;
+      pd->n   = n_task;
+    }else{
+      pd->tab = ptab;
+      pd->n   = n_proc;
+    }
+    // change array indexes to pointers
+    while(n_proc--) ptab[n_proc] = data+(long)(ptab[n_proc]);
+    while(n_task--) ttab[n_task] = data+(long)(ttab[n_task]);
+
+    return pd;
+}
+
