@@ -9,9 +9,11 @@ EXTERN_C_BEGIN
 extern unsigned long long Hertz;   /* clock tick frequency */
 extern long smp_num_cpus;     /* number of CPUs */
 
+#if 0
 #define JT double
-extern void five_cpu_numbers(JT *uret, JT *nret, JT *sret, JT *iret, JT *wret);
+extern void seven_cpu_numbers(JT *uret, JT *nret, JT *sret, JT *iret, JT *wret, JT *xret, JT *yret);
 #undef JT
+#endif
 
 extern int        uptime (double *uptime_secs, double *idle_secs);
 extern void       loadavg(double *av1, double *av5, double *av15);
@@ -53,9 +55,7 @@ extern unsigned long kb_pagetables;
 
 #define BUFFSIZE 8192
 typedef unsigned long long jiff;
-extern void crash(const char *filename);
-extern void getrunners(unsigned int *restrict running, unsigned int *restrict blocked);
-extern void getstat(jiff *restrict cuse, jiff *restrict cice, jiff *restrict csys, jiff *restrict cide, jiff *restrict ciow,
+extern void getstat(jiff *restrict cuse, jiff *restrict cice, jiff *restrict csys, jiff *restrict cide, jiff *restrict ciow, jiff *restrict cxxx, jiff *restrict cyyy,
 	     unsigned long *restrict pin, unsigned long *restrict pout, unsigned long *restrict s_in, unsigned long *restrict sout,
 	     unsigned *restrict intr, unsigned *restrict ctxt,
 	     unsigned int *restrict running, unsigned int *restrict blocked,
@@ -90,5 +90,48 @@ extern unsigned long vm_allocstall;
 
 extern void vminfo(void);
 
+typedef struct disk_stat{
+	unsigned long long reads_sectors;
+	unsigned long long written_sectors;
+	char               disk_name [16];
+	unsigned int       disk_type;
+	unsigned           inprogress_IO;
+	unsigned           merged_reads;
+	unsigned           merged_writes;
+	unsigned           milli_reading;
+	unsigned           milli_spent_IO;
+	unsigned           milli_writing;
+	unsigned           partitions;
+	unsigned           reads;
+	unsigned           weighted_milli_spent_IO;
+	unsigned           writes;
+}disk_stat;
+
+typedef struct partition_stat{
+	unsigned int disk_type;
+	unsigned int partition_num;
+	char partition_name [16];
+	struct disk_stat* parent_disk;
+	unsigned           reads;
+	unsigned long long reads_sectors;
+	unsigned           writes;
+	unsigned           requested_writes;
+}partition_stat;
+
+extern unsigned int getpartitions_num(struct disk_stat *disks, int ndisks);
+extern unsigned int getdiskstat (struct disk_stat**,struct partition_stat**);
+
+typedef struct slab_cache{
+	char name[48];
+	unsigned active_objs;
+	unsigned num_objs;
+	unsigned objsize;
+	unsigned objperslab;
+}slab_cache;
+
+extern unsigned int getslabinfo (struct slab_cache**);
+
+extern unsigned get_pid_digits(void) FUNCTION;
+
 EXTERN_C_END
 #endif /* SYSINFO_H */
