@@ -71,11 +71,41 @@
 
 /******************* PS DEFINE *******************/
 
+// Column flags
+// Justification control for flags field comes first.
+#define CF_JUST_MASK                0x0f
+//      CF_AIXHACK                     0
+#define CF_USER                        1 // left if text, right if numeric
+#define CF_LEFT                        2
+#define CF_RIGHT                       3
+#define CF_UNLIMITED                   4
+#define CF_WCHAN                       5 // left if text, right if numeric
+#define CF_SIGNAL                      6 // right in 9, or 16 if screen_cols>107
+// Then the other flags
+#define CF_CUMUL              0x00000010 // mark cumulative (Summed) headers with 'C' */
+#define CF_PIDMAX             0x00000020 // react to pid_max
+// Only one allowed; use separate bits to catch errors.
+#define CF_PRINT_THREAD_ONLY  0x10000000
+#define CF_PRINT_PROCESS_ONLY 0x20000000
+#define CF_PRINT_EVERY_TIME   0x40000000
+#define CF_PRINT_AS_NEEDED    0x80000000 // means we have no clue, so assume EVERY TIME
+#define CF_PRINT_MASK         0xf0000000
+
 #define needs_for_select (PROC_FILLSTAT | PROC_FILLSTATUS)
 
 /* thread_flags */
-#define TF_show_proc 0x0001  // show the summary line
-#define TF_show_task 0x0002  // show the per-thread lines
+#define TF_B_H         0x0001
+#define TF_B_m         0x0002
+#define TF_U_m         0x0004
+#define TF_U_T         0x0008
+#define TF_U_L         0x0010
+#define TF_show_proc   0x0100  // show the summary line
+#define TF_show_task   0x0200  // show the per-thread lines
+#define TF_show_both   0x0400  // distinct proc/task format lists
+#define TF_loose_tasks 0x0800  // let sorting break up task groups (BSDish)
+#define TF_no_sort     0x1000  // don't know if thread-grouping should survive a sort
+#define TF_no_forest   0x2000  // don't see how to do threads w/ forest option
+#define TF_must_use    0x4000  // options only make sense if LWP/SPID column added
 
 /* personality control flags */
 #define PER_BROKEN_o      0x0001
@@ -115,10 +145,10 @@
 #define FM_c 0x0001 /* -c */
 #define FM_j 0x0002 /* -j */  /* only set when !sysv_j_format */
 #define FM_y 0x0004 /* -y */
-#define FM_L 0x0008 /* -L */
+//#define FM_L 0x0008 /* -L */
 #define FM_P 0x0010 /* -P */
 #define FM_M 0x0020 /* -M */
-#define FM_T 0x0040 /* -T */
+//#define FM_T 0x0040 /* -T */
 #define FM_F 0x0080 /* -F */  /* -F also sets the regular -f flags */
 
 /* sorting & formatting */
@@ -226,13 +256,14 @@ typedef struct sf_node {
 /********************* UNDECIDED GLOBALS **************/
 
 /* output.c */
-extern void show_one_proc(const proc_t *restrict const p);
+extern void show_one_proc(const proc_t *restrict const p, const format_node *restrict fmt);
 extern void print_format_specifiers(void);
 extern const aix_struct *search_aix_array(const int findme);
 extern const shortsort_struct *search_shortsort_array(const int findme);
 extern const format_struct *search_format_array(const char *findme);
 extern const macro_struct *search_macro_array(const char *findme);
 extern void init_output(void);
+extern int pr_nop(char *restrict const outbuf, const proc_t *restrict const pp);
 
 /* global.c */
 extern void reset_global(void);
