@@ -264,8 +264,8 @@ static const char *parse_sysv_option(void){
        * SCO UnixWare uses -L too.
        */
       trace("-L Print LWP (thread) info.\n");
-      thread_flags |= TF_show_task;
-      format_modifiers |= FM_L;
+      thread_flags |= TF_U_L;
+//      format_modifiers |= FM_L;
       break;
     case 'M':  /* someday, maybe, we will have MAC like SGI's Irix */
       trace("-M Print security label for Mandatory Access Control.\n");
@@ -299,8 +299,8 @@ static const char *parse_sysv_option(void){
        * Also, testing shows PID==SPID for all normal processes.
        */
       trace("-T adds strange SPID column (old sproc() threads?)\n");
-      thread_flags |= TF_show_task;
-      format_modifiers |= FM_T;
+      thread_flags |= TF_U_T;
+//      format_modifiers |= FM_T;
       break;
     case 'U': /* end */
       trace("-U select by RUID (supports names).\n");
@@ -315,10 +315,12 @@ static const char *parse_sysv_option(void){
       exclusive("-V");
       display_version();
       exit(0);
+#if 0
     case 'Z':     /* full Mandatory Access Control level info */
       trace("-Z shows full MAC info\n");
       return "Don't understand MAC on Linux.";
       break;
+#endif
     case 'a':
       trace("-a select all with a tty, but omit session leaders.\n");
       simple_select |= SS_U_a;
@@ -358,7 +360,7 @@ static const char *parse_sysv_option(void){
       return "List of session leaders OR effective group IDs was invalid.";
     case 'j':
       trace("-j jobs format.\n");
-      /* Debian uses RD_j and Digital uses JFMT */
+      /* old Debian used RD_j and Digital uses JFMT */
       if(sysv_j_format) format_flags |= FF_Uj;
       else format_modifiers |= FM_j;
       break;
@@ -369,8 +371,7 @@ static const char *parse_sysv_option(void){
     case 'm':
       trace("-m shows threads.\n");
       /* note that AIX shows 2 lines for a normal process */
-      thread_flags |= TF_show_proc;
-      thread_flags |= TF_show_task;
+      thread_flags |= TF_U_m;
       break;
     case 'n': /* end */
       trace("-n sets namelist file.\n");
@@ -442,10 +443,12 @@ static const char *parse_sysv_option(void){
       trace("-y Print lnone info in UID/USER column or do Sun -l hack.\n");
       format_modifiers |= FM_y;
       break;
+#if 0
     case 'z':     /* alias of Mandatory Access Control level info */
       trace("-z shows aliased MAC info\n");
       return "Don't understand MAC on Linux.";
       break;
+#endif
     case '-':
       return "Embedded '-' among SysV options makes no sense.";
       break;
@@ -499,7 +502,7 @@ static const char *parse_bsd_option(void){
 #endif
     case 'H':    // The FreeBSD way (NetBSD:s OpenBSD:k FreeBSD:H  -- NIH???)
       trace("H Print LWP (thread) info.\n");   // was: Use /vmcore as c-dumpfile\n");
-      thread_flags |= TF_show_task;  // FIXME: determine if TF_show_proc is needed
+      thread_flags |= TF_B_H;
       //format_modifiers |= FM_L;    // FIXME: determine if we need something like this
       break;
     case 'L': /* single */
@@ -595,11 +598,11 @@ static const char *parse_bsd_option(void){
       trace("j job control format\n");
       format_flags |= FF_Bj;
       break;
-    case 'k':    // The OpenBSD way (NetBSD:s OpenBSD:k FreeBSD:H  -- NIH???)
+#if 0
+    case 'k':    // OpenBSD: don't hide "kernel threads" -- like the swapper?
       trace("k Print LWP (thread) info.\n");   // was: Use /vmcore as c-dumpfile\n");
-      thread_flags |= TF_show_task;  // FIXME: determine if TF_show_proc is needed
-      //format_modifiers |= FM_L;    // FIXME: determine if we need something like this
       break;
+#endif
     case 'l':
       trace("l Display long format\n");
       format_flags |= FF_Bl;
@@ -614,8 +617,7 @@ static const char *parse_bsd_option(void){
         defer_sf_option("pmem", SF_B_m);
         break;
       }
-      thread_flags |= TF_show_proc;
-      thread_flags |= TF_show_task;
+      thread_flags |= TF_B_m;
       break;
     case 'n':
       trace("n Numeric output for WCHAN, and USER replaced by UID\n");
@@ -1089,6 +1091,43 @@ static void choose_dimensions(void){
   /* perhaps --html and --null should set unlimited width */
 }
 
+static const char *thread_option_check(void){
+  if(!thread_flags){
+    thread_flags = TF_show_proc;
+    return NULL;
+  }
+
+  if(forest_type){
+    return "Thread display conflicts with forest display.";
+  }
+  //thread_flags |= TF_no_forest;
+
+  if((thread_flags&TF_B_H) && (thread_flags&(TF_B_m|TF_U_m)))
+    return "Thread flags conflict; can't use H with m or -m.";
+  if((thread_flags&TF_B_m) && (thread_flags&TF_U_m))
+    return "Thread flags conflict; can't use both m and -m.";
+  if((thread_flags&TF_U_L) && (thread_flags&TF_U_T))
+    return "Thread flags conflict; can't use both -L and -T.";
+
+  if(thread_flags&TF_B_H) thread_flags |= (TF_show_proc|TF_loose_tasks);
+  if(thread_flags&(TF_B_m|TF_U_m)) thread_flags |= (TF_show_proc|TF_show_task|TF_show_both);
+
+  if(thread_flags&(TF_U_T|TF_U_L)){
+    if(thread_flags&(TF_B_m|TF_U_m|TF_B_H)){
+      // Got a thread style, so format modification is a requirement?
+      // Maybe -T/-L has H thread style though. (sorting interaction?)
+      //return "Huh? Tell procps-feedback@lists.sf.net what you expected.";
+      thread_flags |= TF_must_use;
+    }else{
+      // using -L/-T thread style, so format from elsewhere is OK
+      thread_flags |= TF_show_task;  // or like the H option?
+      //thread_flags |= TF_no_sort;
+    }
+  }
+
+  return NULL;
+}
+
 int arg_parse(int argc, char *argv[]){
   const char *err = NULL;
   const char *err2 = NULL;
@@ -1096,15 +1135,12 @@ int arg_parse(int argc, char *argv[]){
   ps_argv = argv;
   thisarg = 0;
 
-#if 0
-  {int debugloop = 0; while(debugloop<argc){
-  trace("argv[%d]=%s\n", debugloop, argv[debugloop]); debugloop++;}}
-#endif
-
   if(personality & PER_FORCE_BSD) goto try_bsd;
 
   err = parse_all_options();
   if(err) goto try_bsd;
+  err = thread_option_check();
+  if(err) goto try_bsd;
   err = process_sf_options(!not_pure_unix);
   if(err) goto try_bsd;
   err = select_bits_setup();
@@ -1133,6 +1169,8 @@ try_bsd:
 
   err2 = parse_all_options();
   if(err2) goto total_failure;
+  err2 = thread_option_check();
+  if(err2) goto total_failure;
   err2 = process_sf_options(!not_pure_unix);
   if(err2) goto total_failure;
   err2 = select_bits_setup();
